@@ -45,7 +45,10 @@ class Body:
     def __init__(
         self,
         name=None,
+        geometry_type=None,
         urdf_file=None,
+        sphere_radius=None,
+        box_half_extent=None,
         device=None,
         use_fixed_base=None,
         use_self_collision=None,
@@ -83,7 +86,10 @@ class Body:
         self._init_callback()
 
         self.name = name
+        self.geometry_type = geometry_type
         self.urdf_file = urdf_file
+        self.sphere_radius = sphere_radius
+        self.box_half_extent = box_half_extent
         self.device = device
         self.use_fixed_base = use_fixed_base
         self.use_self_collision = use_self_collision
@@ -119,6 +125,7 @@ class Body:
         self.dof_target_velocity = dof_target_velocity
         self.dof_force = dof_force
 
+        self.env_ids_reset_base_state = None
         self.env_ids_reset_dof_state = None
 
         self.dof_state = None
@@ -180,6 +187,16 @@ class Body:
         """ """
         self._name = value
 
+    @property
+    def geometry_type(self):
+        """ """
+        return self._geometry_type
+
+    @geometry_type.setter
+    def geometry_type(self, value):
+        """ """
+        self._geometry_type = value
+
     @property
     def urdf_file(self):
         """ """
@@ -190,6 +207,26 @@ class Body:
         """ """
         self._urdf_file = value
 
+    @property
+    def sphere_radius(self):
+        """ """
+        return self._sphere_radius
+
+    @sphere_radius.setter
+    def sphere_radius(self, value):
+        """ """
+        self._sphere_radius = value
+
+    @property
+    def box_half_extent(self):
+        """ """
+        return self._box_half_extent
+
+    @box_half_extent.setter
+    def box_half_extent(self, value):
+        """ """
+        self._box_half_extent = value
+
     @property
     def device(self):
         """ """
@@ -867,6 +904,20 @@ class Body:
                 )
         self._dof_force = value
 
+    @property
+    def env_ids_reset_base_state(self):
+        """ """
+        return self._env_ids_reset_base_state
+
+    @env_ids_reset_base_state.setter
+    def env_ids_reset_base_state(self, value):
+        """ """
+        if value is not None:
+            value = torch.as_tensor(value, dtype=torch.int64, device=self.device)
+            if value.ndim != 1:
+                raise ValueError("'env_ids_reset_base_state' must have a number of dimensions of 1")
+        self._env_ids_reset_base_state = value
+
     @property
     def env_ids_reset_dof_state(self):
         """ """
