@@ -11,7 +11,7 @@ import time
 from contextlib import contextmanager
 
 from easysim.simulators.simulator import Simulator
-from easysim.constants import DoFControlMode
+from easysim.constants import GeometryType, DoFControlMode
 from easysim.contact import create_contact_array
 
 
@@ -42,6 +42,8 @@ class Bullet(Simulator):
 
         if self._cfg.NUM_ENVS != 1:
             raise ValueError("NUM_ENVS must be 1 for Bullet")
+        if self._cfg.SIM_DEVICE != "cpu":
+            raise ValueError("SIM_DEVICE must be 'cpu' for Bullet")
 
         self._connected = False
         self._last_frame_time = 0.0
@@ -98,11 +100,11 @@ class Bullet(Simulator):
         """ """
         try:
             if self._cfg.RENDER:
-                self._p.configureDebugVisualizer(self._p.COV_ENABLE_RENDERING, 0)
+                self._p.configureDebugVisualizer(pybullet.COV_ENABLE_RENDERING, 0)
             yield
         finally:
             if self._cfg.RENDER:
-                self._p.configureDebugVisualizer(self._p.COV_ENABLE_RENDERING, 1)
+                self._p.configureDebugVisualizer(pybullet.COV_ENABLE_RENDERING, 1)
 
     def _load_ground_plane(self):
         """ """
@@ -118,20 +120,66 @@ class Bullet(Simulator):
                     f"For Bullet, 'env_ids_load' must be either None, [] or [0]: '{body.name}'"
                 )
 
-        kwargs = {}
-        if body.use_fixed_base is not None:
-            kwargs["useFixedBase"] = body.use_fixed_base
-        if body.use_self_collision is not None and body.use_self_collision:
-            kwargs["flags"] = self._p.URDF_USE_SELF_COLLISION
         for attr in ("vhacd_enabled", "vhacd_params", "mesh_normal_mode"):
             if getattr(body, attr) is not None:
                 raise ValueError(f"'{attr}' is not supported in Bullet: '{body.name}'")
-        self._body_ids[body.name] = self._p.loadURDF(body.urdf_file, **kwargs)
+        kwargs = {}
+        if body.use_self_collision is not None and body.use_self_collision:
+            kwargs["flags"] = pybullet.URDF_USE_SELF_COLLISION
+        if body.geometry_type is None:
+            raise ValueError(f"For Bullet, 'geometry_type' must not be None: '{body.name}'")
+        if body.geometry_type not in (GeometryType.URDF, GeometryType.SPHERE, GeometryType.BOX):
+            raise ValueError(
+                f"For Bullet, 'geometry_type' only supports URDF, SPHERE, and BOX: '{body.name}'"
+            )
+        if body.geometry_type == GeometryType.URDF:
+            for attr in ("sphere_radius", "box_half_extent"):
+                if getattr(body, attr) is not None:
+                    raise ValueError(f"'{attr}' must be None for geometry type URDF: '{body.name}'")
+            if body.use_fixed_base is not None:
+                kwargs["useFixedBase"] = body.use_fixed_base
+            self._body_ids[body.name] = self._p.loadURDF(body.urdf_file, **kwargs)
+        else:
+            kwargs_visual = {}
+            kwargs_collision = {}
+            if body.geometry_type == GeometryType.SPHERE:
+                for attr in ("urdf_file", "box_half_extent"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"'{attr}' must be None for geometry type SPHERE: '{body.name}'"
+                        )
+                if body.sphere_radius is not None:
+                    kwargs_visual["radius"] = body.sphere_radius
+                    kwargs_collision["radius"] = body.sphere_radius
+                kwargs["baseVisualShapeIndex"] = self._p.createVisualShape(
+                    pybullet.GEOM_SPHERE, **kwargs_visual
+                )
+                kwargs["baseCollisionShapeIndex"] = self._p.createCollisionShape(
+                    pybullet.GEOM_SPHERE, **kwargs_collision
+                )
+            if body.geometry_type == GeometryType.BOX:
+                for attr in ("urdf_file", "sphere_radius"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"'{attr}' must be None for geometry type BOX: '{body.name}'"
+                        )
+                if body.box_half_extent is not None:
+                    kwargs_visual["halfExtents"] = body.box_half_extent
+                    kwargs_collision["halfExtents"] = body.box_half_extent
+                kwargs["baseVisualShapeIndex"] = self._p.createVisualShape(
+                    pybullet.GEOM_BOX, **kwargs_visual
+                )
+                kwargs["baseCollisionShapeIndex"] = self._p.createCollisionShape(
+                    pybullet.GEOM_BOX, **kwargs_collision
+                )
+            if body.use_fixed_base is not None and body.use_fixed_base:
+                kwargs["baseMass"] = 0.0
+            self._body_ids[body.name] = self._p.createMultiBody(**kwargs)
 
         dof_indices = []
         for j in range(self._p.getNumJoints(self._body_ids[body.name])):
             joint_info = self._p.getJointInfo(self._body_ids[body.name], j)
-            if joint_info[2] != self._p.JOINT_FIXED:
+            if joint_info[2] != pybullet.JOINT_FIXED:
                 dof_indices.append(j)
         self._dof_indices[body.name] = np.asanyarray(dof_indices, dtype=np.int64)
 
@@ -139,27 +187,9 @@ class Bullet(Simulator):
 
         # Reset base state.
         if body.initial_base_position is not None:
-            if body.initial_base_position.ndim == 1:
-                self._p.resetBasePositionAndOrientation(
-                    self._body_ids[body.name],
-                    body.initial_base_position[:3],
-                    body.initial_base_position[3:],
-                )
-            if body.initial_base_position.ndim == 2:
-                self._p.resetBasePositionAndOrientation(
-                    self._body_ids[body.name],
-                    body.initial_base_position[0, :3],
-                    body.initial_base_position[0, 3:],
-                )
+            self._reset_base_position(body)
         if body.initial_base_velocity is not None:
-            kwargs = {}
-            if body.initial_base_velocity.ndim == 1:
-                kwargs["linearVelocity"] = body.initial_base_velocity[:3]
-                kwargs["angularVelocity"] = body.initial_base_velocity[3:]
-            if body.initial_base_velocity.ndim == 2:
-                kwargs["linearVelocity"] = body.initial_base_velocity[0, :3]
-                kwargs["angularVelocity"] = body.initial_base_velocity[0, 3:]
-            self._p.resetBaseVelocity(self._body_ids[body.name], **kwargs)
+            self._reset_base_velocity(body)
 
         # Reset DoF state.
         if len(self._dof_indices[body.name]) == 0:
@@ -176,6 +206,32 @@ class Bullet(Simulator):
 
         body.contact_id = [self._body_ids[body.name]]
 
+    def _reset_base_position(self, body):
+        """ """
+        if body.initial_base_position.ndim == 1:
+            self._p.resetBasePositionAndOrientation(
+                self._body_ids[body.name],
+                body.initial_base_position[:3],
+                body.initial_base_position[3:],
+            )
+        if body.initial_base_position.ndim == 2:
+            self._p.resetBasePositionAndOrientation(
+                self._body_ids[body.name],
+                body.initial_base_position[0, :3],
+                body.initial_base_position[0, 3:],
+            )
+
+    def _reset_base_velocity(self, body):
+        """ """
+        kwargs = {}
+        if body.initial_base_velocity.ndim == 1:
+            kwargs["linearVelocity"] = body.initial_base_velocity[:3]
+            kwargs["angularVelocity"] = body.initial_base_velocity[3:]
+        if body.initial_base_velocity.ndim == 2:
+            kwargs["linearVelocity"] = body.initial_base_velocity[0, :3]
+            kwargs["angularVelocity"] = body.initial_base_velocity[0, 3:]
+        self._p.resetBaseVelocity(self._body_ids[body.name], **kwargs)
+
     def _reset_dof_state(self, body):
         """ """
         for i, j in enumerate(self._dof_indices[body.name]):
@@ -234,8 +290,8 @@ class Bullet(Simulator):
                 )
             ):
                 raise ValueError(
-                    "Bullet only supports POSITION_CONTROL, VELOCITY_CONTROL, and TORQUE_CONTROL "
-                    f"modes: '{body.name}'"
+                    "For Bullet, 'dof_control_mode' only supports POSITION_CONTROL, "
+                    f"VELOCITY_CONTROL, and TORQUE_CONTROL: '{body.name}'"
                 )
 
             if (
@@ -245,7 +301,7 @@ class Bullet(Simulator):
                 self._p.setJointMotorControlArray(
                     self._body_ids[body.name],
                     self._dof_indices[body.name],
-                    self._p.VELOCITY_CONTROL,
+                    pybullet.VELOCITY_CONTROL,
                     forces=[0] * len(self._dof_indices[body.name]),
                 )
             if (
@@ -257,7 +313,7 @@ class Bullet(Simulator):
                     self._dof_indices[body.name][
                         body.dof_control_mode == DoFControlMode.TORQUE_CONTROL
                     ],
-                    self._p.VELOCITY_CONTROL,
+                    pybullet.VELOCITY_CONTROL,
                     forces=[0]
                     * len(
                         self._dof_indices[body.name][
@@ -500,8 +556,13 @@ class Bullet(Simulator):
         link_state = [pos + orn + lin + ang]
         if self._num_links[body.name] > 1:
             link_indices = [*range(0, self._num_links[body.name] - 1)]
+            # Need to set computeForwardKinematics=1. See:
+            #     https://github.com/bulletphysics/bullet3/issues/2806
             link_states = self._p.getLinkStates(
-                self._body_ids[body.name], link_indices, computeLinkVelocity=1
+                self._body_ids[body.name],
+                link_indices,
+                computeLinkVelocity=1,
+                computeForwardKinematics=1,
             )
             link_state += [x[4] + x[5] + x[6] + x[7] for x in link_states]
         body.link_state = [link_state]
@@ -561,13 +622,31 @@ class Bullet(Simulator):
                     raise ValueError(f"'{attr}' is not supported in Bullet: '{body.name}'")
 
             if body.env_ids_load is not None and len(body.env_ids_load) == 0:
-                if body.env_ids_reset_dof_state is not None:
-                    raise ValueError(
-                        "For Bullet, 'env_ids_reset_dof_state' should be None if 'env_ids_load' is "
-                        f"set to []: '{body.name}'"
-                    )
+                for attr in ("env_ids_reset_base_state", "env_ids_reset_dof_state"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"For Bullet, '{attr}' should be None if 'env_ids_load' is set to []: "
+                            f"'{body.name}'"
+                        )
                 continue
 
+            if body.env_ids_reset_base_state is not None:
+                if not np.array_equal(body.env_ids_reset_base_state.cpu(), [0]):
+                    raise ValueError(
+                        "For Bullet, 'env_ids_reset_base_state' must be either None or [0]: "
+                        f"'{body.name}'"
+                    )
+                if body.initial_base_position is None and body.initial_base_velocity is None:
+                    raise ValueError(
+                        "'initial_base_position' and 'initial_base_velocity' cannot be both None "
+                        f"when 'env_ids_reset_base_state' is used: {body.name}"
+                    )
+                if body.initial_base_position is not None:
+                    self._reset_base_position(body)
+                if body.initial_base_velocity is not None:
+                    self._reset_base_velocity(body)
+                body.env_ids_reset_base_state = None
+
             if body.attr_array_dirty_flag["link_color"]:
                 self._set_link_color(body)
                 body.attr_array_dirty_flag["link_color"] = False
