@@ -20,7 +20,7 @@ except ImportError:
     from isaacgym import gymtorch, gymutil
 
 from easysim.simulators.simulator import Simulator
-from easysim.constants import DoFControlMode, MeshNormalMode
+from easysim.constants import GeometryType, DoFControlMode, MeshNormalMode
 from easysim.contact import create_contact_array
 
 
@@ -69,15 +69,15 @@ class IsaacGym(Simulator):
             self._sim_device_id = 0
 
         self._device = "cpu"
-        if self._cfg.USE_GPU_PIPELINE:
+        if self._cfg.ISAAC_GYM.USE_GPU_PIPELINE:
             if sim_device_type == "cuda":
                 self._device = "cuda:" + str(self._sim_device_id)
             else:
                 print("GPU pipeline can only be used with GPU simulation. Forcing CPU pipeline.")
-                self._cfg.USE_GPU_PIPELINE = False
+                self._cfg.ISAAC_GYM.USE_GPU_PIPELINE = False
 
-        if not self._cfg.RENDER and self._cfg.GRAPHICS_DEVICE_ID != -1:
-            self._cfg.GRAPHICS_DEVICE_ID = -1
+        if not self._cfg.RENDER and self._cfg.ISAAC_GYM.GRAPHICS_DEVICE_ID != -1:
+            self._cfg.ISAAC_GYM.GRAPHICS_DEVICE_ID = -1
 
         # Support only PhysX for now.
         self._physics_engine = gymapi.SIM_PHYSX
@@ -90,7 +90,9 @@ class IsaacGym(Simulator):
         self._created = False
         self._last_render_time = 0.0
         self._counter_render = 0
-        self._render_time_step = max(1.0 / self._cfg.RENDER_FRAME_RATE, self._cfg.TIME_STEP)
+        self._render_time_step = max(
+            1.0 / self._cfg.ISAAC_GYM.RENDER_FRAME_RATE, self._cfg.TIME_STEP
+        )
         self._render_steps = self._render_time_step / self._cfg.TIME_STEP
 
     def _parse_sim_params(self, cfg, sim_device_type):
@@ -105,11 +107,13 @@ class IsaacGym(Simulator):
             sim_params.substeps = cfg.SUBSTEPS
         sim_params.gravity = gymapi.Vec3(*cfg.GRAVITY)
         sim_params.up_axis = gymapi.UP_AXIS_Z
-        sim_params.use_gpu_pipeline = cfg.USE_GPU_PIPELINE
+        sim_params.use_gpu_pipeline = cfg.ISAAC_GYM.USE_GPU_PIPELINE
 
         sim_params.physx.use_gpu = sim_device_type == "cuda"
-        sim_params.physx.max_depenetration_velocity = cfg.PHYSX.MAX_DEPENETRATION_VELOCITY
-        sim_params.physx.contact_collection = gymapi.ContactCollection(cfg.PHYSX.CONTACT_COLLECTION)
+        sim_params.physx.max_depenetration_velocity = cfg.ISAAC_GYM.PHYSX.MAX_DEPENETRATION_VELOCITY
+        sim_params.physx.contact_collection = gymapi.ContactCollection(
+            cfg.ISAAC_GYM.PHYSX.CONTACT_COLLECTION
+        )
 
         return sim_params
 
@@ -118,7 +122,7 @@ class IsaacGym(Simulator):
         if not self._created:
             self._sim = self._create_sim(
                 self._sim_device_id,
-                self._cfg.GRAPHICS_DEVICE_ID,
+                self._cfg.ISAAC_GYM.GRAPHICS_DEVICE_ID,
                 self._physics_engine,
                 self._sim_params,
             )
@@ -126,7 +130,7 @@ class IsaacGym(Simulator):
             self._load_ground_plane()
             self._load_assets(bodies)
             self._create_envs(
-                self._num_envs, self._cfg.SPACING, int(np.sqrt(self._num_envs)), bodies
+                self._num_envs, self._cfg.ISAAC_GYM.SPACING, int(np.sqrt(self._num_envs)), bodies
             )
             self._cache_and_set_props(bodies)
             self._set_callback(bodies)
@@ -177,8 +181,6 @@ class IsaacGym(Simulator):
         counter_rigid_body = 0
 
         for b, body in enumerate(bodies):
-            asset_root, asset_file = os.path.split(body.urdf_file)
-
             asset_options = gymapi.AssetOptions()
             if body.use_fixed_base is not None:
                 asset_options.fix_base_link = body.use_fixed_base
@@ -212,9 +214,51 @@ class IsaacGym(Simulator):
             if body.mesh_normal_mode is not None:
                 asset_options.mesh_normal_mode = self._MESH_NORMAL_MODE_MAP[body.mesh_normal_mode]
 
-            self._assets[body.name] = self._gym.load_asset(
-                self._sim, asset_root, asset_file, options=asset_options
-            )
+            if body.geometry_type is None:
+                raise ValueError(f"For Isaac Gym, 'geometry_type' must not be None: '{body.name}'")
+            if body.geometry_type not in (GeometryType.URDF, GeometryType.SPHERE, GeometryType.BOX):
+                raise ValueError(
+                    f"For Isaac Gym, 'geometry_type' only supports URDF and SPHERE: '{body.name}'"
+                )
+            if body.geometry_type == GeometryType.URDF:
+                for attr in ("sphere_radius", "box_half_extent"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"'{attr}' must be None for geometry type URDF: '{body.name}'"
+                        )
+                asset_root, asset_file = os.path.split(body.urdf_file)
+                self._assets[body.name] = self._gym.load_asset(
+                    self._sim, asset_root, asset_file, options=asset_options
+                )
+            if body.geometry_type == GeometryType.SPHERE:
+                for attr in ("urdf_file", "box_half_extent"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"'{attr}' must be None for geometry type SPHERE: '{body.name}'"
+                        )
+                if body.sphere_radius is None:
+                    raise ValueError(
+                        "For Isaac Gym, 'sphere_radius' must not be None if 'geometry_type' is set "
+                        f"to SPHERE: '{body.name}'"
+                    )
+                self._assets[body.name] = self._gym.create_sphere(
+                    self._sim, body.sphere_radius, options=asset_options
+                )
+            if body.geometry_type == GeometryType.BOX:
+                for attr in ("urdf_file", "sphere_radius"):
+                    if getattr(body, attr) is not None:
+                        raise ValueError(
+                            f"'{attr}' must be None for geometry type BOX: '{body.name}'"
+                        )
+                if body.box_half_extent is None:
+                    raise ValueError(
+                        "For Isaac Gym, 'box_half_extent' must not be None if 'geometry_type' is "
+                        f"set to BOX: '{body.name}'"
+                    )
+                self._assets[body.name] = self._gym.create_box(
+                    self._sim, *[x * 2 for x in body.box_half_extent], options=asset_options
+                )
+
             self._asset_num_dofs[body.name] = self._gym.get_asset_dof_count(self._assets[body.name])
             self._asset_num_rigid_bodies[body.name] = self._gym.get_asset_rigid_body_count(
                 self._assets[body.name]
@@ -396,7 +440,7 @@ class IsaacGym(Simulator):
 
             if body.link_color is None:
                 # Avoid error from `get_rigid_body_color()` when `graphics_device` is set to -1.
-                if self._cfg.GRAPHICS_DEVICE_ID == -1:
+                if self._cfg.ISAAC_GYM.GRAPHICS_DEVICE_ID == -1:
                     body.link_color = [
                         [[1.0, 1.0, 1.0]] * self._asset_num_rigid_bodies[body.name]
                     ] * self._num_envs
@@ -699,26 +743,7 @@ class IsaacGym(Simulator):
             )
 
         for body in bodies:
-            if body.initial_base_position is None:
-                initial_base_position = self._initial_actor_root_state[
-                    self._actor_root_indices[body.name], :7
-                ]
-            else:
-                if body.env_ids_load is None or body.initial_base_position.ndim == 1:
-                    initial_base_position = body.initial_base_position
-                else:
-                    initial_base_position = body.initial_base_position[body.env_ids_load]
-            self._actor_root_state[self._actor_root_indices[body.name], :7] = initial_base_position
-            if body.initial_base_velocity is None:
-                initial_base_velocity = self._initial_actor_root_state[
-                    self._actor_root_indices[body.name], 7:
-                ]
-            else:
-                if body.env_ids_load is None or body.initial_base_velocity.ndim == 1:
-                    initial_base_velocity = body.initial_base_velocity
-                else:
-                    initial_base_velocity = body.initial_base_velocity[body.env_ids_load]
-            self._actor_root_state[self._actor_root_indices[body.name], 7:] = initial_base_velocity
+            self._reset_base_state_buffer(body)
 
             if self._asset_num_dofs[body.name] == 0:
                 for attr in ("initial_dof_position", "initial_dof_velocity"):
@@ -757,6 +782,29 @@ class IsaacGym(Simulator):
 
         self._check_and_update_props(bodies, env_ids=env_ids)
 
+    def _reset_base_state_buffer(self, body):
+        """ """
+        if body.initial_base_position is None:
+            initial_base_position = self._initial_actor_root_state[
+                self._actor_root_indices[body.name], :7
+            ]
+        else:
+            if body.env_ids_load is None or body.initial_base_position.ndim == 1:
+                initial_base_position = body.initial_base_position
+            else:
+                initial_base_position = body.initial_base_position[body.env_ids_load]
+        self._actor_root_state[self._actor_root_indices[body.name], :7] = initial_base_position
+        if body.initial_base_velocity is None:
+            initial_base_velocity = self._initial_actor_root_state[
+                self._actor_root_indices[body.name], 7:
+            ]
+        else:
+            if body.env_ids_load is None or body.initial_base_velocity.ndim == 1:
+                initial_base_velocity = body.initial_base_velocity
+            else:
+                initial_base_velocity = body.initial_base_velocity[body.env_ids_load]
+        self._actor_root_state[self._actor_root_indices[body.name], 7:] = initial_base_velocity
+
     def _reset_dof_state_buffer(self, body):
         """ """
         if body.initial_dof_position is None:
@@ -831,8 +879,8 @@ class IsaacGym(Simulator):
                             np.isin(np.nonzero(body.attr_array_dirty_mask[attr])[0], env_ids.cpu())
                         ):
                             raise ValueError(
-                                f"For Isaac Gym, to change '{attr}' for some env also requires the env "
-                                f"indices to be in `env_ids`: '{body.name}'"
+                                f"For Isaac Gym, to change '{attr}' for some env also requires the "
+                                f"env indices to be in `env_ids`: '{body.name}'"
                             )
                         mask |= body.attr_array_dirty_mask[attr]
                 env_ids_masked = np.nonzero(mask)[0]
@@ -870,8 +918,8 @@ class IsaacGym(Simulator):
                                 )
                             ):
                                 raise ValueError(
-                                    f"For Isaac Gym, to change '{attr}' for certain env also requires "
-                                    f"the env index to be in `env_ids`: '{body.name}'"
+                                    f"For Isaac Gym, to change '{attr}' for certain env also "
+                                    f"requires the env index to be in `env_ids`: '{body.name}'"
                                 )
                             mask |= body.attr_array_dirty_mask[attr]
                     env_ids_masked = np.nonzero(mask)[0]
@@ -906,10 +954,26 @@ class IsaacGym(Simulator):
 
         self._check_and_update_props(bodies)
 
+        reset_base_state = False
         reset_dof_state = False
-        actor_indices = []
+        actor_indices_base = []
+        actor_indices_dof = []
 
         for b, body in enumerate(bodies):
+            if body.env_ids_reset_base_state is not None:
+                if body.env_ids_load is not None and not torch.all(
+                    torch.isin(body.env_ids_reset_base_state, body.env_ids_load)
+                ):
+                    raise ValueError(
+                        "'env_ids_reset_base_state' must be a subset of 'env_ids_load' for "
+                        f"non-None 'env_ids_load': '{body.name}'"
+                    )
+                self._reset_base_state_buffer(body)
+                if not reset_base_state:
+                    reset_base_state = True
+                actor_indices_base.append(self._actor_indices[body.env_ids_reset_base_state, b])
+                body.env_ids_reset_base_state = None
+
             if self._asset_num_dofs[body.name] == 0:
                 for attr in (
                     "dof_target_position",
@@ -934,7 +998,7 @@ class IsaacGym(Simulator):
                 self._reset_dof_state_buffer(body)
                 if not reset_dof_state:
                     reset_dof_state = True
-                actor_indices.append(self._actor_indices[body.env_ids_reset_dof_state, b])
+                actor_indices_dof.append(self._actor_indices[body.env_ids_reset_dof_state, b])
                 body.env_ids_reset_dof_state = None
 
             if body.dof_target_position is not None and (
@@ -1091,8 +1155,17 @@ class IsaacGym(Simulator):
                         body.dof_control_mode == DoFControlMode.TORQUE_CONTROL,
                     ] = dof_force
 
+        if reset_base_state:
+            actor_indices = torch.cat(actor_indices_base)
+            self._gym.set_actor_root_state_tensor_indexed(
+                self._sim,
+                gymtorch.unwrap_tensor(self._actor_root_state),
+                gymtorch.unwrap_tensor(actor_indices),
+                len(actor_indices),
+            )
+
         if reset_dof_state:
-            actor_indices = torch.cat(actor_indices)
+            actor_indices = torch.cat(actor_indices_dof)
             self._gym.set_dof_state_tensor_indexed(
                 self._sim,
                 gymtorch.unwrap_tensor(self._dof_state),
